import wx
import serial


class HeaterPanel(wx.Panel):

    def __init__(self, parent, pwrbox, mainframe, data, type, *args, **kw):

        super().__init__(parent)

        self.parent = parent
        self.pwrbox = pwrbox
        self.data = data
        self.targettime = 0
        self.timestamp = 0
        self.main_sizer = wx.BoxSizer(wx.VERTICAL)
        self.type = type
        self.mode = []
        self.pwm = [0, 0]
        self.gain = [0, 0]
        self.min = [0, 0]
        self.ofs = [0, 0]
        self.pwm_windows = []
        self.auto_windows = []
        self.pwm_object = []
        self.auto_object = []
        self.min_object = []
        self.gain_object = []
        self.ofs_object = []
        self.amb_temperature = 0.0
        self.humidity = 0.0
        self.dewpoint = 0.0
        self.data = []
        self.mainframe = mainframe

        self.Size = (327, 128)

        self.Bind(wx.EVT_CLOSE, self.OnClose)

        for field in data:

            idx = field['index']
            self.mode.append('manual')
            self.data.append(field)

            pnl = wx.Panel(self, size=(375, 35))
            small_sizer = wx.BoxSizer(wx.HORIZONTAL)

            item = wx.CheckBox(pnl, wx.ID_ANY, field['name'] + ":" +
                               field['auto'], size=(200, 35))
            self.auto_object.append(item)
            field['id1'] = item.GetId()
            item.Bind(wx.EVT_CHECKBOX, self.OnCheck1)
            small_sizer.Add(item, proportion=0, flag=wx.ALL, userData=field)

            pnl.SetSizer(small_sizer)
            self.main_sizer.Add(pnl, wx.ALL)

            pnl = wx.Panel(self, size=(375, 35))
            small_sizer = wx.BoxSizer(wx.HORIZONTAL)

            item = wx.StaticText(pnl, id=wx.ID_ANY, label=" PWM", size=(35, -1))
            small_sizer.Add(item, 1, wx.ALIGN_CENTER_VERTICAL)

            item = wx.SpinCtrl(pnl, id=wx.ID_ANY, min=0, max=255,
                               initial=0, size=(125, 35))
            self.pwm_object.append(item)
            field['id2'] = item.GetId()
            if idx == 0:
                item.Bind(wx.EVT_TEXT, self.OnPWM0)
            else:
                item.Bind(wx.EVT_TEXT, self.OnPWM1)
            small_sizer.Add(item, proportion=1, flag=wx.ALL, userData=field)

            item = wx.StaticText(pnl, id=wx.ID_ANY, label=" Gain", size=(35, -1))
            small_sizer.Add(item, 1, wx.ALIGN_CENTER_VERTICAL)
            gain = field['gain']
            item = wx.SpinCtrl(pnl, id=wx.ID_ANY, min=0, max=255, initial=gain,
                               size=(125, 35))
            self.gain_object.append(item)
            if idx == 0:
                item.Bind(wx.EVT_TEXT, self.OnGain0)
            else:
                item.Bind(wx.EVT_TEXT, self.OnGain1)
            small_sizer.Add(item, proportion=0, flag=wx.ALL, userData=field)
            # small_sizer.Add(0, 0, proportion=1)

            pnl.SetSizer(small_sizer)
            self.main_sizer.Add(pnl, wx.ALL)

            pnl = wx.Panel(self, size=(375, 35))
            small_sizer = wx.BoxSizer(wx.HORIZONTAL)

            item = wx.StaticText(pnl, id=wx.ID_ANY, label=" Min", size=(35, -1))
            small_sizer.Add(item, 1, wx.ALIGN_CENTER_VERTICAL)

            min = field['min']
            item = wx.SpinCtrl(pnl, id=wx.ID_ANY, min=0, max=255, initial=min,
                               size=(125, 35))

            self.min_object.append(item)
            if idx == 0:
                item.Bind(wx.EVT_TEXT, self.OnMin0)
            else:
                item.Bind(wx.EVT_TEXT, self.OnMin1)
            small_sizer.Add(item, proportion=1, flag=wx.ALL, userData=field)

            item = wx.StaticText(pnl, id=wx.ID_ANY, label=" Ofs", size=(35, -1))
            small_sizer.Add(item, 1, wx.ALIGN_CENTER_VERTICAL)

            offset = field['offset']
            item = wx.SpinCtrl(pnl, id=wx.ID_ANY, min=0, max=10,
                               initial=offset, size=(125, 35))

            self.ofs_object.append(item)
            if idx == 0:
                item.Bind(wx.EVT_TEXT, self.OnOfs0)
            else:
                item.Bind(wx.EVT_TEXT, self.OnOfs1)
            small_sizer.Add(item, proportion=1, flag=wx.ALL, userData=field)

            pnl.SetSizer(small_sizer)
            self.main_sizer.Add(pnl, wx.ALL)

        self.SetSizer(self.main_sizer)

    def update_gui(self, data):
        self.data = data
        for field in data:
            idx = field['index']
            obj = self.auto_object[idx]
            if obj:
                htr_no = str(field['index'])
                if field['auto'] == "Auto":
                    obj.SetValue(True)
                    self.pwm_object[idx].Disable()
                    obj.SetLabelText(data[idx]["name"] + " : Auto")
                else:
                    obj.SetValue(False)
                    self.pwm_object[idx].Enable()
                    obj.SetLabelText(data[idx]["name"] + " : Manual")

            self.pwm_object[idx].SetValue(field['pwm'])
            self.min_object[idx].SetValue(field['min'])
            self.gain_object[idx].SetValue(field['gain'])

    def _find_data(self, obj):
        _id = obj.GetId()
        items = self.data
        match = 0
        data = None
        try:
            data = next(item for item in items if item["id1"] == _id)
            match = 1
        except StopIteration:
            try:
                data = next(item for item in items if item["id2"] == _id)
                match = 2
            except StopIteration:
                pass
        return data, match

    def OnCheck1(self, event):
        obj = event.EventObject
        data, match = self._find_data(obj)

        if data['type'] == 'htr':
            if match == 1:
                idx = data['index']
                if obj.IsChecked():
                    if self.pwrbox.set_auto_htr(idx, 1):
                        obj.Label = data['name'] + " : Auto"
                        self.mode[idx] = "auto"
                        self.pwm_object[idx].Disable()
                    else:
                        obj.SetValue(False)

                else:
                    self.pwrbox.set_auto_htr(idx, 0)
                    obj.Label = data['name'] + " : Manual"
                    self.pwrbox.set_htr_pwm(idx, self.pwm[idx])
                    self.mode[idx] = "manual"
                    self.pwm_object[idx].Enable()

    def OnPWM0(self, event):
        if self.mode[0] == 'manual':

            self.pwm[0] = event.EventObject.GetValue()
            self.pwrbox.set_htr_pwm(0, self.pwm[0])
        else:
            event.EventObject.Disable()

    def OnPWM1(self, event):
        if self.mode[1] == 'manual':
            self.pwm[1] = event.EventObject.GetValue()
            self.pwrbox.set_htr_pwm(1, self.pwm[1])

    def OnGain0(self, event):
        self.gain[0] = event.EventObject.GetValue()
        self.pwrbox.set_htr_gain(0, self.gain[0])

    def OnGain1(self, event):
        self.gain[1] = event.EventObject.GetValue()
        self.pwrbox.set_htr_gain(1, self.gain[1])

    def OnMin0(self, event):
        self.min[0] = event.EventObject.GetValue()
        self.pwrbox.set_htr_min(0, self.min[0])

    def OnMin1(self, event):
        self.min[1] = event.EventObject.GetValue()
        self.pwrbox.set_htr_min(1, self.min[1])

    def OnOfs0(self, event):
        self.ofs[0] = event.EventObject.GetValue()
        self.pwrbox.set_htr_ofs(0, self.ofs[0])

    def OnOfs1(self, event):
        self.ofs[1] = event.EventObject.GetValue()
        self.pwrbox.set_htr_ofs(1, self.ofs[1])

    def OnTimer(self):
        try:
            tmp = self.pwrbox.get_temperature()
            self.amb_temperature = tmp

            tmp = self.pwrbox.get_humidity()
            self.humidity = tmp

            tmp = self.pwrbox.get_dewpoint()
            self.dewpoint = tmp

        except serial.serialutil.SerialException as e:
            print("Serial port gone")
            return

        self.pwm[0] = self.pwrbox.get_htr_pwm(0)
        self.pwm[1] = self.pwrbox.get_htr_pwm(1)

        self.mainframe.statusbar.SetStatusText(
            u"T = {:.2f}\u00b0C    RH = {:.2f}%    DP = {:.2f}\u00b0C". format(
                self.amb_temperature,
                self.humidity,
                self.dewpoint).encode('utf-8'), 1)

        self.pwm_object[0].SetValue(self.pwm[0])
        self.pwm_object[1].SetValue(self.pwm[1])

    def OnClose(self, event):
        pass
