import wx


class MetricsPanel(wx.Panel):

    def __init__(self, parent, pwrbox, pwrdata, htrdata, *args, **kw):

        super().__init__(parent)

        self.parent = parent
        self.pwrbox = pwrbox
        self.main_sizer = wx.FlexGridSizer(10, 4, 6, 25)
        self.main_sizer.AddGrowableCol(3, proportion=0)
        font = wx.Font(9, wx.DEFAULT, wx.NORMAL, wx.BOLD)
        lbl = wx.StaticText(self, label="Device")
        lbl.SetFont(font)
        amps = wx.StaticText(self, label="Amps")
        amps.SetFont(font)
        ah = wx.StaticText(self, label="AH")
        ah.SetFont(font)
        watts = wx.StaticText(self, -1, "Watts", (0, 0), (50, -1), wx.ALIGN_RIGHT)
        watts.SetFont(font)
        self.main_sizer.Add(lbl, proportion=1, flag=wx.EXPAND)
        self.main_sizer.Add(amps, proportion=1, flag=wx.EXPAND)
        self.main_sizer.Add(ah, proportion=1, flag=wx.EXPAND)
        self.main_sizer.Add(watts, proportion=1, flag=wx.EXPAND)

        for field in pwrdata:

            lbl = wx.StaticText(self, label=field['name'])
            lbl.SetFont(font)
            amps = wx.StaticText(self, label="0.00")
            amps.SetFont(font)
            ah = wx.StaticText(self, label="0.00")
            ah.SetFont(font)
            watts = wx.StaticText(self, -1, "0.00", (0, 0), (50, -1), wx.ALIGN_RIGHT)
            watts.SetFont(font)
            self.main_sizer.Add(lbl, proportion=1, flag=wx.EXPAND, userData=field)
            self.main_sizer.Add(amps, proportion=1, flag=wx.EXPAND)
            self.main_sizer.Add(ah, proportion=1, flag=wx.EXPAND)
            self.main_sizer.Add(watts, proportion=1, flag=wx.EXPAND)

        for field in htrdata:

            lbl = wx.StaticText(self, label=field['name'])
            lbl.SetFont(font)
            amps = wx.StaticText(self, label="0.00")
            amps.SetFont(font)
            ah = wx.StaticText(self, label="0.00")
            ah.SetFont(font)
            watts = wx.StaticText(self, -1, "0.00", (0, 0), (50, -1), wx.ALIGN_RIGHT)
            watts.SetFont(font)
            self.main_sizer.Add(lbl, proportion=1, flag=wx.EXPAND, userData=field)
            self.main_sizer.Add(amps, proportion=1, flag=wx.EXPAND)
            self.main_sizer.Add(ah, proportion=1, flag=wx.EXPAND)
            self.main_sizer.Add(watts, proportion=1, flag=wx.EXPAND)

        lbl = wx.StaticText(self, label="Total")
        lbl.SetFont(font)
        amps = wx.StaticText(self, label="0.00")
        amps.SetFont(font)
        ah = wx.StaticText(self, label="0.00")
        ah.SetFont(font)
        watts = wx.StaticText(self, -1, "0.00", (0, 0), (50, -1), wx.ALIGN_RIGHT)
        watts.SetFont(font)
        self.main_sizer.Add(lbl, proportion=1, flag=wx.EXPAND)
        self.main_sizer.Add(amps, proportion=1, flag=wx.EXPAND)
        self.main_sizer.Add(ah, proportion=1, flag=wx.EXPAND)
        self.main_sizer.Add(watts, proportion=1, flag=wx.EXPAND)
        self.main_sizer.SetFlexibleDirection(wx.HORIZONTAL)
        self.top_level_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.top_level_sizer.Add(self.main_sizer, proportion=1, flag=wx.ALL | wx.EXPAND, border=10)
        self.right_sizer = wx.FlexGridSizer(10, 2, 6, 25)
        lbl1 = wx.StaticText(self, label="Volts")
        lbl1.SetFont(font)
        self.volts = wx.StaticText(self, label="0.00")
        lbl2 = wx.StaticText(self, label="AH Total")
        lbl2.SetFont(font)
        self.aht = wx.StaticText(self, label="0.00")
        lbl3 = wx.StaticText(self, label="Runtime")
        lbl3.SetFont(font)
        self.runtime = wx.StaticText(self, label="000:00:00")

        self.right_sizer.Add(lbl1, border=10)
        self.right_sizer.Add(self.volts)
        self.right_sizer.Add(lbl2, border=10)
        self.right_sizer.Add(self.aht)
        self.right_sizer.Add(lbl3, border=10)
        self.right_sizer.Add(self.runtime)
        self.top_level_sizer.Add(self.right_sizer, proportion=1, flag=wx.ALL | wx.EXPAND, border=10)
        self.SetSizer(self.top_level_sizer)

        # self.timer = wx.Timer(self)
        # self.Bind(wx.EVT_TIMER, self.OnTimer, self.timer)
        # self.timer.Start(10000)

    def get_object(self, index, offset):
        ptr = index * 4 + 4 + offset
        item = self.main_sizer.GetItem(ptr)
        obj = item.GetWindow()
        return obj

    def update_gui(self):

        volts = self.pwrbox.get_volts()
        runtime = self.pwrbox.get_run_time()

        for index in range(0, 6):

            item = self.main_sizer.GetItem(index * 4 + 4)
            # obj = item.GetWindow()
            # txt = obj.LabelText
            val = self.pwrbox.get_pwr_amps(index)
            obj = self.get_object(index, 1)
            obj.SetLabel("%3.2f" % val)
            new_val = self.pwrbox.get_pah(index)
            obj = self.get_object(index, 2)
            obj.SetLabel("%3.2f" % new_val)
            obj = self.get_object(index, 3)
            new_val = "%3.2f" % (val * volts)
            obj.SetLabel(new_val)

        for index in range(0, 2):

            base_ptr = index + 6
            item = self.main_sizer.GetItem(base_ptr)
            obj = item.GetWindow()
            val = self.pwrbox.get_htr_amps(index)
            obj = self.get_object(base_ptr, 1)
            obj.SetLabel("%3.2f" % val)
            new_val = self.pwrbox.get_hah(index)
            obj = self.get_object(base_ptr, 2)
            obj.SetLabel("%3.2f" % new_val)
            obj = self.get_object(base_ptr, 3)
            new_val = "%3.2f" % (val * volts)
            obj.SetLabel(str(new_val))

        base_ptr = 8
        item = self.main_sizer.GetItem(base_ptr)
        obj = item.GetWindow()
        val = self.pwrbox.get_cur_amps()
        obj = self.get_object(base_ptr, 1)
        obj.SetLabel(str(val))
        new_val = self.pwrbox.get_ah_total()
        obj = self.get_object(base_ptr, 2)
        obj.SetLabel(str(new_val))
        self.aht.SetLabel(str(new_val))
        obj = self.get_object(base_ptr, 3)
        new_val = "%3.2f" % (val * volts)
        obj.SetLabel(str(new_val))

        new_val = "%3.2f" % (volts)
        self.volts.SetLabel(new_val)
        self.runtime.SetLabel(runtime)

    def OnTimer(self, event):

        self.update_gui()
